import bzrlib
import os
import os.path
import sys
import tempfile
import shutil

def temp_branch():
    dirname = tempfile.mkdtemp("temp-branch")
    return bzrlib.Branch(dirname, init=True)

def rm_branch(br):
    shutil.rmtree(br.base)

def is_clean(cur_branch):
    """
    Return true if no files are modifed or unknown
    >>> br = temp_branch()
    >>> is_clean(br)
    True
    >>> fooname = os.path.join(br.base, "foo")
    >>> file(fooname, "wb").write("bar")
    >>> is_clean(br)
    False
    >>> bzrlib.add.smart_add([fooname])
    >>> is_clean(br)
    False
    >>> br.commit("added file")
    >>> is_clean(br)
    True
    >>> rm_branch(br)
    """
    old_tree = cur_branch.basis_tree()
    new_tree = cur_branch.working_tree()
    for file_state, fid, old_name, new_name, kind in \
        bzrlib.diff_trees(old_tree, new_tree):
        if file_state not in ('I', '.'):
            return False
    return True

def set_pull_data(br, location, rev_id):
    pull_file = file (br.controlfilename("x-pull-data"), "wb")
    pull_file.write("%s\n%s\n" % (location, rev_id))

def get_pull_data(br):
    """
    >>> br = temp_branch()
    >>> get_pull_data(br)
    (None, None)
    >>> set_pull_data(br, 'http://somewhere', '888-777')
    >>> get_pull_data(br)
    ('http://somewhere', '888-777')
    >>> rm_branch(br)
    """
    filename = br.controlfilename("x-pull-data")
    if not os.path.exists(filename):
        return (None, None)
    pull_file = file (filename, "rb")
    location, rev_id = [f.rstrip('\n') for f in pull_file]
    return location, rev_id

"""
>>> shell_escape('hello')
'\h\e\l\l\o'
"""
def shell_escape(arg):
    return "".join(['\\'+c for c in arg])

def safe_system(args):
    """
    >>> real_system = os.system
    >>> os.system = sys.stdout.write
    >>> safe_system(['a', 'b', 'cd'])
    \\a \\b \\c\\d
    >>> os.system = real_system
    """
    arg_str = " ".join([shell_escape(a) for a in args])
    return os.system(arg_str)

def rsync(source, target):
    """
    >>> real_system = os.system
    >>> os.system = sys.stdout.write
    >>> rsync("a", "b")
    \\r\\s\\y\\n\\c \\-\\a\\v \\-\\-\\d\\e\\l\\e\\t\\e \\a \\b
    >>> os.system = real_system
    """
    safe_system(("rsync", "-av", "--delete", source, target))

def pull(cur_branch, location=None):
    pull_location, pull_revision = get_pull_data(cur_branch)
    if pull_location is not None:
        if cur_branch.last_patch() != pull_revision:
            print "Aborting: This branch has had commits, so pull would lose data."
            sys.exit(1)
    if len(sys.argv) > 1:
        pull_location = sys.argv[1] 
        if not pull_location.endswith('/'):
            pull_location.append('/')

    if pull_location is None:
        print "No pull location saved.  Please specify one on the command line."
        sys.exit(1)

    if not is_clean(cur_branch):
        print "Error: This tree has uncommitted changes or unknown (?) files."
        sys.exit(1)

    print "Synchronizing with %s" % pull_location
    rsync (pull_location, cur_branch.base)

    set_pull_data(cur_branch, pull_location, cur_branch.last_patch())

def run_tests():
    import doctest
    bzrlib.trace.create_tracefile([])
    result = doctest.testmod()
    if result[1] > 0:
        if result[0] == 0:
            print "All tests passed"
    else:
        print "No tests to run"
if __name__ == "__main__":
    run_tests()
